@@ -22,27 +22,30 @@ class BuilderSIGMA:
         self.model_data: DataModel = input_model_data
         self.roxie_data: pd.APIdata = input_roxie_data
         self.settings_dict: dict = settings_dict
-        self.output_path: str = output_path
-
-        self.g = ws.GatewaySIGMA()
-        self.a = ws.ArraysSIGMA
-
-        self.conductor_name = str
-        self.cableParameters = pd.CondPar
-        # self.wedge_elements =
-        # self.model =
-        self.coil = []
-        self.elements = {}
-        self.coil_areas = []
-        self.wedge_areas = []
-        self.show_halfTurns = True  # to be provided by the yaml in the future
+        if output_path:
+            self.output_path: str = output_path
+        else:
+            self.output_path = os.getcwd()
 
         if (not self.model_data or not self.roxie_data) and flag_build:
             raise Exception('Cannot build model without providing DataModel and RoxieData')
         elif flag_build:
+            self.g = ws.GatewaySIGMA()
+            self.a = ws.ArraysSIGMA
+
+            self.max_r: float = 0.0
+            self.conductor_name = str
+            self.cableParameters = pd.CondPar
+            # self.wedge_elements =
+            # self.model =
+            self.coil = []
+            self.elements = {}
+            self.coil_areas = []
+            self.wedge_areas = []
+            self.show_halfTurns = True  # to be provided by the yaml in the future
             self.model_name = f"{self.model_data.GeneralParameters.magnet_name}_Model"
 
-            self.bh_curve_database = os.path.join(Path(__file__).parent.parent.parent,
+            self.bh_curve_database = os.path.join(Path(__file__).parent.parent.parent.parent,
                                                   "steam_models", "magnets", "bh_curve_database.txt")
 
             self.COMSOL_exe_path = self.settings_dict['comsolexe_path']
@@ -51,10 +54,10 @@ class BuilderSIGMA:
             self.COMSOL_compile_path = f"{self.COMSOL_exe_path[:-4]}compile.exe"
             self.COMSOL_batch_path = f"{self.COMSOL_exe_path[:-4]}batch.exe"
 
-            self.model_java_file_path_save = f"{os.path.join(self.output_path, self.model_name)}.java"
+            self.model_java_file_path = f"{os.path.join(self.output_path, self.model_name)}.java"
             self.compile_batch_file_path = f"{os.path.join(self.output_path, self.model_name)}_Compile_and_Open.bat"
-            self.model_java_file_path_bat = f"{os.path.join(self.output_path, self.model_name)}.java"
             self.model_class_file_path = f"{os.path.join(self.output_path, self.model_name)}.class"
+            self.split_java_file_path = []
 
             self.build_magnet()
             self.save_model_java()
@@ -62,6 +65,8 @@ class BuilderSIGMA:
             #self.plot_magnet()
 
     def build_magnet(self):
+        self.air_ff_domain()
+        self.air_domain()
         self.coil_domain()
         self.iron_yoke_domain()
         self.wedge_domain()
@@ -71,7 +76,50 @@ class BuilderSIGMA:
         ar2 = area.mirrorY()
         return area, ar2, ar2.mirrorX(), area.mirrorX()
 
+    def air_ff_domain(self):
+        """
+            Air far field domain
+        """
+        iron = self.roxie_data.iron
+
+        kpc = self.g.Point.ofCartesian(0.0, 0.0)
+        for i in iron.key_points:
+            max_i = max(iron.key_points[i].x, iron.key_points[i].y)
+            if max_i > self.max_r:
+                self.max_r = max_i
+
+        kp1 = self.g.Point.ofCartesian(self.max_r * 2 * 0.95, 0.0)
+        kp2 = self.g.Point.ofCartesian(0.0, self.max_r * 2 * 0.95)
+        kp1_out = self.g.Point.ofCartesian(self.max_r * 2, 0.0)
+        kp2_out = self.g.Point.ofCartesian(0.0, self.max_r * 2)
+        ln1 = self.g.Line.ofEndPoints(kpc, kp1_out)
+        ln2 = self.g.Arc.ofEndPointsCenter(kp1_out, kp2_out, kpc)
+        ln3 = self.g.Line.ofEndPoints(kp2_out, kp2)
+        ln4 = self.g.Arc.ofEndPointsCenter(kp2, kp1, kpc)
+
+        hyper_areas = self.mirrorXY(self.g.Area.ofHyperLines(
+            self.a.create_hyper_line_array(self.g.gateway, tuple([ln1, ln2, ln3, ln4]))))
+
+        arg = []  # elements
+        for i, ar in enumerate(hyper_areas):
+            arg.append(self.g.Element(f"AFF_El{i}", ar))
+
+        self.air_far_field = self.a.create_element_array(self.g.gateway, tuple(arg))
+
+    def air_domain(self):
+        """
+            Air domain
+        """
+        kpc = self.g.Point.ofCartesian(0.0, 0.0)
+
+        self.air = self.a.create_element_array(self.g.gateway, self.g.Element('Air', self.g.Area.ofHyperLines(
+            self.a.create_hyper_line_array(self.g.gateway,
+                                           self.g.Circumference.ofCenterRadius(kpc, self.max_r * 2 * 0.95)))))
+
     def iron_yoke_domain(self):
+        """
+            Iron yoke and internal air domains
+        """
         iron = self.roxie_data.iron
 
         keyPointsCOMSOL = {}
@@ -116,6 +164,9 @@ class BuilderSIGMA:
             self.elements[i] = self.a.create_element_array(self.g.gateway, tuple(arg))
 
     def cable_domain(self):
+        """
+            Cable characteristics and conductor properties
+        """
         self.g.cable.setLabel(self.conductor_name)
         self.g.cable.setwInsulNarrow(self.cableParameters.wInsulNarrow)
         self.g.cable.setwInsulWide(self.cableParameters.wInsulWide)
@@ -153,6 +204,9 @@ class BuilderSIGMA:
         self.g.cable.setResitivityCopperFit(self.g.Cable.ResitivityCopperFitEnum.rho_Cu_CUDI)
 
     def coil_domain(self):
+        """
+            Winding blocks domain
+        """
         poles = ()
         for coil_nr, coil in self.roxie_data.coil.coils.items():
             for pole_nr, pole in coil.poles.items():
@@ -197,6 +251,9 @@ class BuilderSIGMA:
     # self.coil_areas = self.coil_areas + coil_areas_mirrored
 
     def wedge_domain(self):
+        """
+            Inter-block wedges domain
+        """
         wedges = self.roxie_data.wedges
 
         elements = []
@@ -220,6 +277,9 @@ class BuilderSIGMA:
         self.wedge_elements = self.a.create_element_array(self.g.gateway, tuple(elements))
 
     def save_model_java(self):
+        domains = [self.g.AirFarFieldDomain("AFF", self.g.MatDatabase.MAT_AIR, self.air_far_field)]
+        domains += [self.g.AirDomain("AIR", self.g.MatDatabase.MAT_AIR, self.air)]
+
         orderedElements = list(self.elements)
         # change order of generated domains to override correctly
         for hyper_hole_key, hyper_hole in self.roxie_data.iron.hyper_holes.items():
@@ -234,7 +294,7 @@ class BuilderSIGMA:
         self.iron_yoke = [
             self.g.IronDomain(i, self.bh_curve_database, self.roxie_data.iron.hyper_areas[i].material, self.elements[i])
             for i in orderedElements]  # domains
-        domains = self.iron_yoke
+        domains += self.iron_yoke
 
         domains.append(self.g.CoilDomain("CO", self.g.MatDatabase.MAT_COIL, self.coil))
 
@@ -243,7 +303,7 @@ class BuilderSIGMA:
         # Create magnet model
         cfg = self.g.ConfigSigma()
 
-        cfg.setOutputModelPath(self.model_java_file_path_save)
+        cfg.setOutputModelPath(self.model_java_file_path)
         cfg.setExternalCFunLibPath(self.settings_dict['CFunLibPath'])
 
         srv = self.g.TxtSigmaServer(cfg.getOutputModelPath(), cfg.getComsolBatchPath())
@@ -253,7 +313,7 @@ class BuilderSIGMA:
         self.model.buildMPH(self.a.create_domain_array(self.g.gateway, tuple(domains)))
         self.model.save()
 
-        with open(self.model_java_file_path_save) as java_file:
+        with open(self.model_java_file_path) as java_file:
             max_no_lines = 6e4
             public_indexes = []
             files = []
@@ -276,7 +336,7 @@ class BuilderSIGMA:
                 file_path = os.path.join(self.output_path, self.model_name)
                 files += [open(file_path + '_%d.java' % i, 'w')]
                 name = self.model_name + '_%d' % i
-                # self.split_java_file_path_save += [f"{os.path.join(self.bat_path, self.model_name + '_%d' % i)}.java"]
+                self.split_java_file_path += [f"{os.path.join(self.output_path, self.model_name + '_%d' % i)}"]
                 files[i].writelines(content[0:public_indexes[0] - 2] + ['public class ' + name + ' {\n', '\n'])
                 if i == 0:
                     files[i].writelines('\tpublic static Model run1(Model mph) {\n')
@@ -290,7 +350,7 @@ class BuilderSIGMA:
                     additional_lines[name] = {'start': no_run[i - 1], 'end': no_run[i] - 1}
                 files[i].close()
 
-        with open(self.model_java_file_path_save, 'w') as java_file:
+        with open(self.model_java_file_path, 'w') as java_file:
             content = content[0:public_indexes[0] + 2] + ['\n'] + \
                       content[public_indexes[-1]:public_indexes[-1] + 2] + ['\t}\n'] + ['}\n']
             content.insert(public_indexes[0] + 2, '\t\tmph = ' + self.model_name + '_0.run1(mph);\n')
@@ -350,8 +410,16 @@ class BuilderSIGMA:
         # print(df)
 
     def save_compile_and_open_bat(self):
-        script_lines = [
-            f'"{self.COMSOL_compile_path}" -jdkroot "{self.java_jdk_path}" "{self.model_java_file_path_bat}"',
+        script_lines = []
+        class_paths = ''
+        for file in self.split_java_file_path:
+            script_lines += [f'"{self.COMSOL_compile_path}" -jdkroot "{self.java_jdk_path}" "{file}.java"',
+                             f'"{self.java_jdk_path}\\bin\\jar.exe" cf "{file}.jar" "{file}.class"']
+            class_paths += f'-classpathadd "{file}.jar" '
+
+        script_lines += [
+            f'"{self.COMSOL_compile_path}" -jdkroot "{self.java_jdk_path}" {class_paths}'
+            f'"{self.model_java_file_path}"',
             f'"{self.COMSOL_batch_path}" -inputfile "{self.model_class_file_path}" '
             f'-outputfile "{os.path.join(self.output_path, self.model_data.GeneralParameters.magnet_name)}.mph"',
             f'"{self.COMSOL_exe_path}" -open "{os.path.join(self.output_path, self.model_name)}.mph"']
@@ -361,26 +429,34 @@ class BuilderSIGMA:
 
 
 if __name__ == "__main__":
-    from parsers.ParserRoxie import ParserRoxie
+    from steam_sdk.parsers.ParserRoxie import ParserRoxie
 
-    magnet_name = 'MQXF_V2'  # 'MQY_2in1'  #
+    magnet_name = 'MQY_2in1'  #'MQXF_V2'  # 'MBX'  #
     current_path = Path(__file__).parent
-    path = Path.joinpath(current_path.parent, 'tests', 'builders', 'input', magnet_name)
+    path = Path.joinpath(current_path.parent.parent, 'tests', 'builders', 'input', magnet_name)
     file_model_data = Path.joinpath(path, 'modelData_' + magnet_name + '.yaml')
-    with open(file_model_data, "r") as stream:
-        dictionary_yaml = yaml.safe_load(stream)
-        model_data = DataModel(**dictionary_yaml)
-    path_iron = Path.joinpath(path, model_data.Sources.iron_fromROXIE)
-    path_data = Path.joinpath(path, model_data.Sources.coil_fromROXIE)
-    path_cadata = Path.joinpath(path.parent, model_data.Sources.conductor_fromROXIE)
+    if Path.exists(file_model_data):
+        with open(file_model_data, "r") as stream:
+            dictionary_yaml = yaml.safe_load(stream)
+            model_data = DataModel(**dictionary_yaml)
+        path_iron = Path.joinpath(path, model_data.Sources.iron_fromROXIE)
+        path_data = Path.joinpath(path, model_data.Sources.coil_fromROXIE)
+    else:
+        path_iron = None
+        path_data = Path.joinpath(path, f"{magnet_name}.data")
+    path_cadata = Path.joinpath(path.parent, 'roxie.cadata')
 
     roxie_parser = ParserRoxie()
     roxie_data = roxie_parser.getData(dir_data=path_data, dir_cadata=path_cadata, dir_iron=path_iron)
     # with open('../tests/parsers/references/cadata_REFERENCE.yaml', 'w') as yaml_file:
     #   yaml.dump(roxie_data.conductor.dict(), yaml_file, default_flow_style=False, sort_keys=False)
+    for environ_var in ['HOMEPATH', 'SWAN_HOME']:
+        if environ_var in os.environ:
+            user_name = os.path.basename(os.path.normpath(os.environ[environ_var]))
+    path_sdk = Path(__file__).parent.parent.parent.parent
+    path_settings = Path.joinpath(path_sdk, "steam_models", "settings", "user_settings")
+    if Path.exists(Path.joinpath(path_settings, f"settings.{user_name}.yaml")):
+        with open(Path.joinpath(path_settings, f"settings.{user_name}.yaml"), 'r') as stream:
+            settings = yaml.safe_load(stream)
 
-    bs = BuilderSIGMA(input_model_data=model_data, input_roxie_data=roxie_data)
-    bs.build_magnet()
-    bs.save_model_java()
-    bs.save_compile_and_open_bat()
-    bs.plot_magnet()
+    bs = BuilderSIGMA(settings_dict=settings, input_model_data=model_data, input_roxie_data=roxie_data)
