@@ -46,7 +46,7 @@ class General(BaseModel):
     circuit_name: str = None
     model: Model = Model()
     T_initial: float = None               # T00 (LEDET), Top (SIGMA)
-    magnetic_length: float = None   # l_magnet (LEDET), magLength (SIGMA)
+    magnetic_length: float = None   # l_magnet (LEDET), magLength (SIGMA), magneticLength (ProteCCT)
     magnet_inductance: MagnetInductance = MagnetInductance()
 
 
@@ -490,7 +490,7 @@ class GeometryGenerationOptionsProteCCT(BaseModel):
     numColumnStrands: int = None
     IcFactor: float = None
     polyimideToEpoxyRatio: float = None
-    windingOrder: List[int] = []
+    windingOrder: str = None  # Note: it will be written in a single cell in the ProteCCT file
 
 
 class PhysicsProteCCT(BaseModel):
@@ -531,7 +531,7 @@ class PlotsProteCCT(BaseModel):
         Level 2: Class for ProteCCT plots options
     """
     withPlots: int = None
-    plotPauseTime: int = None
+    plotPauseTime: float = None
 
 
 class PostProcessingProteCCT(BaseModel):
@@ -539,7 +539,7 @@ class PostProcessingProteCCT(BaseModel):
         Level 2: Class for ProteCCT post-processing options
     """
     withVoltageEvaluation: int = None
-    voltageToGroundOutputSelection: List[int] = []
+    voltageToGroundOutputSelection: str = None  # Note: it will be written in a single cell in the ProteCCT file
 
 
 class PROTECCT(BaseModel):
