@@ -1,14 +1,19 @@
+import os
+from pathlib import Path
 import numpy as np
 import datetime
-import openpyxl
 import shutil
+import json
+import openpyxl
 from openpyxl.cell.cell import Cell
 from openpyxl.styles import Font
 from openpyxl.utils import get_column_letter
-from pathlib import Path
 
 from steam_sdk.builders.BuilderLEDET import BuilderLEDET
+from steam_sdk.data.TemplateLEDET import get_template_LEDET_inputs_sheet, get_template_LEDET_options_sheet, get_template_LEDET_plots_sheet, get_template_LEDET_variables_sheet
 from steam_sdk.parsers import ParserMap2d
+from steam_sdk.parsers.ParserExcel import read_row, compare_two_parameters, write2Excel
+from steam_sdk.utils.NumpyEncoder import NumpyEncoder
 
 
 class ParserLEDET:
@@ -50,7 +55,7 @@ class ParserLEDET:
             try:
                 if (attribute == None):
                     if worksheetInputs.cell(i, 3).value is not None:
-                        values = self.read_row(worksheetInputs, i)
+                        values = read_row(worksheetInputs, i)
                         values = np.array([k for k in values if(str(k))])
                         current = builder_ledet.getAttribute(builder_ledet.Inputs, lastAttribute)
                         current = np.vstack((current, values))
@@ -59,7 +64,7 @@ class ParserLEDET:
                         continue
                 elif (type(builder_ledet.getAttribute(builder_ledet.Inputs, attribute)) == np.ndarray):
                     lastAttribute = attribute
-                    values = self.read_row(worksheetInputs, i)
+                    values = read_row(worksheetInputs, i)
                     values = np.array([k for k in values if(str(k))])
                     builder_ledet.setAttribute(builder_ledet.Inputs, attribute, values)
                 else:
@@ -76,7 +81,7 @@ class ParserLEDET:
             attribute = worksheetOptions.cell(i, 2).value
             try:
                 if (type(builder_ledet.getAttribute(builder_ledet.Options, attribute)) == np.ndarray):
-                    values = self.read_row(worksheetOptions, i)
+                    values = read_row(worksheetOptions, i)
                     values = np.array([k for k in values if(str(k))])
                     builder_ledet.setAttribute(builder_ledet.Options, attribute, values)
                 else:
@@ -93,7 +98,7 @@ class ParserLEDET:
             attribute = worksheetPlots.cell(i, 2).value
             try:
                 if (type(builder_ledet.getAttribute(builder_ledet.Plots, attribute)) == np.ndarray):
-                    values = self.read_row(worksheetPlots, i, St=True)[2:]
+                    values = read_row(worksheetPlots, i, St=True)[2:]
                     values = np.array([k for k in values if(str(k))])
                     builder_ledet.setAttribute(builder_ledet.Plots, attribute, values)
                 else:
@@ -113,8 +118,8 @@ class ParserLEDET:
                 attribute = worksheetVariables.cell(i, 2).value
                 try:
                     if (type(builder_ledet.getAttribute(builder_ledet.Variables, attribute)) == np.ndarray):
-                        if attribute != 'typeVariableToSaveTxt':  values = self.read_row(worksheetVariables, i, St = True)[2:]
-                        else:  values = self.read_row(worksheetVariables, i)
+                        if attribute != 'typeVariableToSaveTxt':  values = read_row(worksheetVariables, i, St = True)[2:]
+                        else:  values = read_row(worksheetVariables, i)
                         values = np.array([k for k in values if(str(k))])
                         builder_ledet.setAttribute(builder_ledet.Variables, attribute, values)
                     else:
@@ -130,29 +135,118 @@ class ParserLEDET:
 
         return builder_ledet
 
-    def read_row(self, workSheet, Nrow, St = False):
+
+    def writeLedet2Excel(self, full_path_file_name: str, verbose: bool = False, SkipConsistencyCheck: bool = True):
         '''
-            *** Function that reads a row of a worksheet of an Excel file ***
+        ** Writes a LEDET input file **
 
-            :param workSheet: Name of the worksheet
-            :type workSheet: str
-            :param Nrow: Number of the row
-            :type Nrow: int
-            :param St: Flag indicating whether the entry to read is a string
-            :type St: bool
-
-            :return: np.array([])
+        :param full_path_file_name:
+        :param verbose:
+        :param SkipConsistencyCheck:
+        :return:
         '''
-        rowValues = np.array([])
-        row = workSheet[Nrow]
-        for cell in row:
-            if not St:
-                if isinstance(cell.value, str): continue
-            rowValues = np.append(rowValues, cell.value)
-        rowValues = rowValues[rowValues != None]
-        return rowValues
 
-    def write2Excel(self, nameFileLEDET: str, verbose: bool = True, SkipConsistencyCheck: bool = False):
+        self._expandCoolingScalarToArray(verbose)
+
+        # Import templates for LEDET input file sheets
+        template_LEDET_inputs_sheet    = get_template_LEDET_inputs_sheet()
+        template_LEDET_options_sheet   = get_template_LEDET_options_sheet()
+        template_LEDET_plots_sheet     = get_template_LEDET_plots_sheet()
+        template_LEDET_variables_sheet = get_template_LEDET_variables_sheet()
+
+        # Define optional variables, which will be written only if present in the dataclass
+        optional_variables_input_sheet = [
+            'alpha_Nb3Sn0_inGroup',
+            'f_scaling_Jc_BSCCO2212_inGroup',
+            'overwrite_f_internalVoids_inGroup', 'overwrite_f_externalVoids_inGroup',
+            'f_RRR1_Cu_inGroup', 'f_RRR2_Cu_inGroup', 'f_RRR3_Cu_inGroup',
+            'RRR1_Cu_inGroup', 'RRR2_Cu_inGroup', 'RRR3_Cu_inGroup',
+            ]
+
+        ### Inputs sheet
+        name_sheet_inputs = 'Inputs'  # This defines the name of the sheet and also of the variable group
+        LEDET_inputs_sheet = []
+        for row in template_LEDET_inputs_sheet:
+            name, description = row[0], row[2]
+
+            # If the row defines an empty row, or a title row, leave it unchanged
+            if name == None:
+                LEDET_inputs_sheet.append(row)  # Leave the row unchanged
+                continue  # stop treating this row: go to the next row
+
+            # Get value of the current parameter
+            value = self.builder_ledet.getAttribute(name_sheet_inputs, name)
+
+            # Skip optional variables if they have 0 elements
+            if name in optional_variables_input_sheet:
+                print(value)
+                print(type(value))
+                if isinstance(value, list) and len(value) == 0:
+                    print('Variable {} is optional and has 0 elements, hence it will be skipped.'.format(name))
+                    continue
+                elif isinstance(value, np.ndarray) and value.shape[0] == 0:
+                    print('Variable {} is optional and has 0 elements, hence it will be skipped.'.format(name))
+                    continue
+                elif len(value) == 1 and value is None:
+                    print('Variable {} is optional and has 0 elements, hence it will be skipped.'.format(name))
+                    continue
+
+            # Assign value to the variable sheet
+            LEDET_inputs_sheet.append([name, value, description])
+
+        ### Options sheet
+        name_sheet_options = 'Options'  # This defines the name of the sheet and also of the variable group
+        LEDET_options_sheet = []
+        for row in template_LEDET_options_sheet:
+            name, description = row[0], row[2]
+
+            # If the row defines an empty row, or a title row, leave it unchanged
+            if name == None:
+                LEDET_options_sheet.append(row)  # Leave the row unchanged
+                continue  # stop treating this row: go to the next row
+
+            # Assign value to the variable sheet
+            value = self.builder_ledet.getAttribute(name_sheet_options, name)
+            LEDET_options_sheet.append([name, value, description])
+
+        ### Plots sheet
+        name_sheet_plots = 'Plots'  # This defines the name of the sheet and also of the variable group
+        LEDET_plots_sheet = []
+        for row in template_LEDET_plots_sheet:
+            name, description = row[0], row[2]
+
+            # If the row defines an empty row, or a title row, leave it unchanged
+            if name == None:
+                LEDET_plots_sheet.append(row)  # Leave the row unchanged
+                continue  # stop treating this row: go to the next row
+
+            # Assign value to the variable sheet
+            value = self.builder_ledet.getAttribute(name_sheet_plots, name)
+            LEDET_plots_sheet.append([name, value, description])
+
+        ### Variables sheet
+        name_sheet_variables = 'Variables'  # This defines the name of the sheet and also of the variable group
+        LEDET_variables_sheet = []
+        for row in template_LEDET_variables_sheet:
+            name, description = row[0], row[2]
+
+            # If the row defines an empty row, or a title row, leave it unchanged
+            if name == None:
+                LEDET_variables_sheet.append(row)  # Leave the row unchanged
+                continue  # stop treating this row: go to the next row
+
+            # Assign value to the variable sheet
+            value = self.builder_ledet.getAttribute(name_sheet_variables, name)
+            LEDET_variables_sheet.append([name, value, description])
+
+        # Write LEDET Excel input file
+        write2Excel(name_file=full_path_file_name,
+                    name_sheets=[name_sheet_inputs, name_sheet_options, name_sheet_plots, name_sheet_variables],
+                    listOf_variables_values_descriptions=[LEDET_inputs_sheet, LEDET_options_sheet, LEDET_plots_sheet, LEDET_variables_sheet],
+                    verbose=verbose)
+
+
+    def writeLedet2Excel_OLD(self, nameFileLEDET: str, verbose: bool = True, SkipConsistencyCheck: bool = False):
         '''
             *** Function that writes a LEDET input file as an Excel file ***
 
@@ -186,24 +280,24 @@ class ParserLEDET:
         if verbose:
             print('')
             print('### Write "Variables" sheet ###')
-        self.writeWorkbook(workbook, "Variables", builder_ledet.Variables, builder_ledet.descriptionsVariables)
+        self.writeLedetWorkbookSheet(workbook, "Variables", builder_ledet.Variables, builder_ledet.descriptionsVariables)
 
         if verbose:
             print('')
             print('### Write "Plots" sheet ###')
-        self.writeWorkbook(workbook, "Plots", builder_ledet.Plots, builder_ledet.descriptionsPlots)
+        self.writeLedetWorkbookSheet(workbook, "Plots", builder_ledet.Plots, builder_ledet.descriptionsPlots)
 
         if verbose:
             print('')
             print('### Write "Options" sheet ###')
-        self.writeWorkbook(workbook, "Options", builder_ledet.Options, builder_ledet.descriptionsOptions)
+        self.writeLedetWorkbookSheet(workbook, "Options", builder_ledet.Options, builder_ledet.descriptionsOptions)
 
         # Special case: if cooling 0 int, expand 0 to all half-turns in the written file. If done, will revert afterwards
         self._expandCoolingScalarToArray(verbose)
         if verbose:
             print('')
             print('### Write "Inputs" sheet ###')
-        self.writeWorkbook(workbook, "Inputs", builder_ledet.Inputs, builder_ledet.descriptionsInputs)
+        self.writeLedetWorkbookSheet(workbook, "Inputs", builder_ledet.Inputs, builder_ledet.descriptionsInputs)
 
         # Save the workbook
         std = workbook['Sheet']
@@ -222,7 +316,7 @@ class ParserLEDET:
             print('New file ' + nameFileLEDET + ' generated.')
         return
 
-    def writeWorkbook(self, book, sheet, variableGroup, variableLabels):
+    def writeLedetWorkbookSheet(self, book, sheet, variableGroup, variableLabels):
         """
             **Write one sheet of a LEDET input file**
 
@@ -363,80 +457,56 @@ class ParserLEDET:
             if verbose: print('sim3D_f_cooling_right is a scalar. Expanding value to all half-turns')
             self.builder_ledet.Inputs.sim3D_f_cooling_right = np.array([self.builder_ledet.Inputs.sim3D_f_cooling_right] * int(np.sum(self.builder_ledet.Inputs.nT)))
 
-    def copy_map2d(self, magnet_name, verbose: bool = False):
-        ''' rename and copy '''
 
-        ### new naming of file (depending on magnet properties)
+    def write2json(self, full_path_file_name: str, verbose: bool = True, SkipConsistencyCheck: bool = False):
+        '''
+            *** Function that writes a LEDET input file as a .json file ***
 
-        if self.builder_ledet.Options.flagIron == 1:
-            pre_suffix = "_WithIron"
+            Function to write a LEDET input file containing "Inputs", "Options", "Plots", and "Variables" variables
+
+            :param full_path_file_name: String defining the name of the LEDET input file to be written
+            :type full_path_file_name: string
+            :param verbose: flag that determines whether the output are printed
+            :type verbose: bool
+            :param SkipConsistencyCheck: flag that determines, whether the parameters shall be checked for consistency or not [False = Apply checks, True = Skip checks]
+            :type SkipConsistencyCheck: bool
+
+            :return: None
+        '''
+
+        # Unpack variables
+        builder_ledet = self.builder_ledet
+
+        # Consistency check
+        if not SkipConsistencyCheck:
+            if self._consistencyCheckLEDET():
+                print("Variables are not consistent! Writing aborted - ", full_path_file_name)
+                return
+            else:
+                if verbose: print("Preliminary consistency check was successful! - ", full_path_file_name)
         else:
-            pre_suffix = "_NoIron"
+            print("Skipped consistency checks.")
 
-        if self.builder_ledet.Options.flagSelfField == 1:
-            pre_pre_suffix = "WithSelfField"
-        else:
-            pre_pre_suffix = "NoSelfFiled"
+        # Write LEDET data into a dictionary
+        ledet_data_dict = {
+            **builder_ledet.Inputs.__dict__,
+            **builder_ledet.Options.__dict__,
+            **builder_ledet.Plots.__dict__,
+            **builder_ledet.Variables.__dict__}  # to add program-specific variables
 
-        suffix = "_All" + pre_suffix + pre_pre_suffix
+        # Write output .json file
+        with open(full_path_file_name, 'w') as outfile:
+            json.dump(ledet_data_dict, outfile, cls=NumpyEncoder)
 
-        # Paths
-        path_sdk = Path(__file__).parent.parent.parent
-        path_ROXIE_input = Path(path_sdk, "tests", "builders", "input", magnet_name)
-        path_magnet_output = Path(path_sdk, "tests", "builders", "output", magnet_name)
-        file_name_new = magnet_name + suffix  + "test" + ".map2d"
+        # Display time stamp
+        currentDT = datetime.datetime.now()
+        if verbose:
+            print(' ')
+            print('Time stamp: ' + str(currentDT))
+            print('New file ' + full_path_file_name + ' generated.')
+        return
 
-        # Copy
-        shutil.copy2(Path(path_ROXIE_input, magnet_name + ".map2d"), Path(path_magnet_output, file_name_new))
-
-        return file_name_new
-
-    def copy_modified_map2d_ribbon_cable(self, magnet_name, geometry_ribbon_cable, verbose: bool = False):
-        ''' #TODO '''
-
-        ### Create path of required magnet files
-        # Paths
-        path_sdk = Path(__file__).parent.parent.parent
-        path_magnet_output = Path(path_sdk, "tests", "builders", "output", magnet_name)
-        path_ROXIE_input = Path(path_sdk, "tests", "builders", "input", magnet_name)
-
-        ### naming of file (depending on magnet properties)
-
-        # if self.builder_ledet.Options.flagIron == 1:
-        #     pre_suffix = "_WithIron"
-        # else:
-        #     pre_suffix = "_NoIron"
-        #
-        # if self.builder_ledet.Options.flagSelfField == 1:
-        #     pre_pre_suffix = "_WithSelfField"
-        # else:
-        #     pre_pre_suffix = "_NoSelfField"
-
-        pre_suffix = "_WithIron"
-        pre_pre_suffix = "_WithSelfField"
-        suffix = "_All" + pre_suffix + pre_pre_suffix
-        file_name_new = magnet_name + suffix + ".map2d"
-
-        # Get name and data of (old) ROXIE file
-        file_name_ROXIE = magnet_name + ".map2d"
-        file_directory_ROXIE = Path(path_ROXIE_input, file_name_ROXIE)
-        content_ROXIE_file = open(file_directory_ROXIE, "r").read().split("\n")
-
-        # Parse to ParserMap2d
-        NewMap2d = ParserMap2d.modify_map2d_ribbon_cable(file_directory_ROXIE, geometry_ribbon_cable)
-        #
-        ParserMap2d.multiply_column_by_value(NewMap2d, 3, 1000)
-        ParserMap2d.multiply_column_by_value(NewMap2d, 4, 1000)
-        ParserMap2d.multiply_column_by_value(NewMap2d, 7, 1000000)
-        # Create .map2d file from ParserMap2d
-        ParserMap2d.create_map2d_file(NewMap2d, content_ROXIE_file[0], Path(path_magnet_output, file_name_new),
-                                      file_name_new)
-
-        return file_name_new
-
-
-###############################################################
-# START - Methods for consistency checks of LEDET input files #
+#######################  Methods for consistency checks of LEDET input files - START  #######################
 # TODO: Update consistency checks of LEDET input files to the latest LEDET features
 
     def __getNumberOfCoilSections(self):
@@ -467,7 +537,7 @@ class ParserLEDET:
             Check if Inductance matrix is squared. Issues a warning if it is not the size of the number of turns. Returns result as bool
 
             :param Turns: Number of turns of the LEDET object
-            :type arr: int
+            :type Turns: int
             :return: bool
         '''
         if type(self.Inputs.M_InductanceBlock_m) != np.ndarray:
@@ -495,7 +565,7 @@ class ParserLEDET:
             Check if Helium options are both set and have the correct and same size, returns bool of result
 
             :param Groups: Number of groups of the LEDET object
-            :type arr: int
+            :type Groups: int
             :return: bool
         '''
         k = self.Inputs.overwrite_f_externalVoids_inGroup
@@ -767,12 +837,11 @@ class ParserLEDET:
         if not self.__checkM_InductanceBlock_m(int(sum(self.Inputs.nT)/2)):
             Break = 1
         return Break
+#######################  Methods for consistency checks of LEDET input files - END  #######################
 
-# END - Methods for consistency checks of LEDET input files #
-###############################################################
-# START - Helper functions #
 
-def CompareLEDETParameters(FileA, FileB, Precision=1E-5, showIndices=0):
+#######################  Helper functions - START  #######################
+def CompareLEDETParameters(fileA, fileB, max_relative_error=1E-5, show_indices=0):
     '''
         Compare all the variables imported from two LEDET Excel input files
     '''
@@ -780,112 +849,113 @@ def CompareLEDETParameters(FileA, FileB, Precision=1E-5, showIndices=0):
     Diff = 0
 
     pl_a = ParserLEDET(BuilderLEDET(flag_build=False))
-    pl_a.readFromExcel(FileA, verbose=False)
+    pl_a.readFromExcel(fileA, verbose=False)
     pl_b = ParserLEDET(BuilderLEDET(flag_build=False))
-    pl_b.readFromExcel(FileB, verbose=False)
-    print("Starting Comparison of A: ({}) and B: ({})".format(FileA, FileB))
+    pl_b.readFromExcel(fileB, verbose=False)
+    print("Starting Comparison of A: ({}) and B: ({})".format(fileA, fileB))
 
     ## Check Inputs
     for attribute in pl_a.builder_ledet.Inputs.__annotations__:
-        Diff = _compare_two_parameters(pl_a.builder_ledet.getAttribute("Inputs", attribute),
-                                       pl_b.builder_ledet.getAttribute("Inputs", attribute),
-                                       Diff, attribute, Precision, showIndices)
+        Diff = compare_two_parameters(pl_a.builder_ledet.getAttribute("Inputs", attribute),
+                                      pl_b.builder_ledet.getAttribute("Inputs", attribute),
+                                      Diff, attribute, max_relative_error, show_indices)
 
     ## Check Options
     for attribute in pl_a.builder_ledet.Options.__annotations__:
-        Diff = _compare_two_parameters(pl_a.builder_ledet.getAttribute("Options", attribute),
-                                       pl_b.builder_ledet.getAttribute("Options", attribute),
-                                       Diff, attribute, Precision, showIndices)
+        Diff = compare_two_parameters(pl_a.builder_ledet.getAttribute("Options", attribute),
+                                      pl_b.builder_ledet.getAttribute("Options", attribute),
+                                      Diff, attribute, max_relative_error, show_indices)
 
     ## Check Plots
     for attribute in pl_a.builder_ledet.Plots.__annotations__:
-        Diff = _compare_two_parameters(pl_a.builder_ledet.getAttribute("Plots", attribute),
-                                       pl_b.builder_ledet.getAttribute("Plots", attribute),
-                                       Diff, attribute, Precision, showIndices)
+        Diff = compare_two_parameters(pl_a.builder_ledet.getAttribute("Plots", attribute),
+                                      pl_b.builder_ledet.getAttribute("Plots", attribute),
+                                      Diff, attribute, max_relative_error, show_indices)
 
     ## Check Variables
     for attribute in pl_a.builder_ledet.Variables.__annotations__:
-        Diff = _compare_two_parameters(pl_a.builder_ledet.getAttribute("Variables", attribute),
-                                       pl_b.builder_ledet.getAttribute("Variables", attribute),
-                                       Diff, attribute, Precision, showIndices)
+        Diff = compare_two_parameters(pl_a.builder_ledet.getAttribute("Variables", attribute),
+                                      pl_b.builder_ledet.getAttribute("Variables", attribute),
+                                      Diff, attribute, max_relative_error, show_indices)
 
     if Diff == 0:
-        print("Files {} and {} are equal.".format(FileA,FileB))
+        print("Files {} and {} are equal.".format(fileA, fileB))
         return True
     else:
         return False
 
 
-def _compare_two_parameters(var_1, var_2, Diff, attribute, Precision, showIndices):
-    '''
-        Helper-function for BuilderLEDET() to compare to parameters
-        Precision is a relative value
-    '''
-    Block = 1
-    CC = 0
+def copy_map2d(magnet_name: str, map2d_file_name: str, output_path: str, flagIron: int, flagSelfField: int,
+               suffix: str = '', verbose: bool = False):
+    ''' rename and copy '''
 
-    if isinstance(var_1, float) or isinstance(var_1, int):
-        if abs(var_1 - var_2) > Precision * abs(var_1):
-            if Block: print(
-                "Found difference in scalar Parameter {}, A: {}, B: {}".format(attribute, var_1, var_2))
-            Block = 0
-            Diff = True
+    # Checks if path exists
+    if output_path != '' and not os.path.isdir(output_path):
+        print("Output folder {} does not exist. Making it now".format(output_path))
+        Path(output_path).mkdir(parents=True)
 
-    elif var_1 is None or var_2 is None:
-        if var_1 is not None or var_2 is not None:
-            Diff = 1
-            print('Parameter A is {} while B is {}'.format(var_1, var_2))
+    ### new naming of file (depending on magnet properties)
 
-    elif len(var_1) != len(var_2):
-        Diff = 1
-        if Block:
-            Block = 0
-            print('Parameter {} of A, {} has not the same length as Parameter of B, {}'.format(attribute,
-                                                                                               len(var_1),
-                                                                                               len(var_2)))
+    if flagIron == 1:
+        suffix_iron = "_WithIron"
     else:
-        Pos = []
-        for k in range(len(var_1)):
-            try:
-                if var_1[k] != var_2[k]:
-                    if isinstance(var_1[k], str):
-                        if var_1[k] != var_2[k]:
-                            Diff = 1
-                            if Block:
-                                print("Found difference in vector Parameter {}".format(attribute))
-                                Block = 0
-                            Pos.append(k)
+        suffix_iron = "_NoIron"
 
-                    elif abs(var_1[k] - var_2[k]) > Precision * abs(var_1[k]):
-                        Diff = 1
-                        if Block:
-                            print("Found difference in vector Parameter {}".format(attribute))
-                            Block = 0
-                        Pos.append(k)
-            except:
-                for j in range(var_1.shape[1]):
-                    if var_1[k, j] != var_2[k, j]:
-                        if isinstance(var_1[k], str):
-                            Diff = 1
-                            if Block:
-                                print("Found difference in matrix Parameter {}".format(attribute))
-                                Block = 0
-                            Pos.append([k, j])
-                        elif abs(var_1[k, j] - var_2[k, j]) > Precision * abs(var_1[k, j]):
-                            Diff = 1
-                            if Block:
-                                print("Found difference in matrix Parameter {}".format(attribute))
-                                Block = 0
-                            Pos.append([k, j])
+    if flagSelfField == 1:
+        suffix_self = "_WithSelfField"
+    else:
+        suffix_self = "_NoSelfField"
 
-        if len(Pos) > 0:
-            if len(Pos) < 10:
-                print("Different Positions: {}".format(Pos))
-            else:
-                print("Many values are different (>10)")
-                if showIndices: print(Pos)
+    suffix_complete = suffix + suffix_iron + suffix_self
+    file_name_output = magnet_name + suffix_complete + ".map2d"
 
-    return Diff
+    # Paths
+    output_path_full = os.path.join(output_path, file_name_output)
 
-# END - Helper functions #
-###############################################################
+    # Copy
+    shutil.copy2(map2d_file_name, output_path_full)
+
+    return file_name_output
+
+
+def copy_modified_map2d_ribbon_cable(magnet_name: str, map2d_file_name: str, output_path: str, geometry_ribbon_cable,
+                                     flagIron: int, flagSelfField: int, suffix: str = '', verbose: bool = False):
+    ''' #TODO '''
+
+    ### Check if output path exists
+
+    if output_path != '' and not os.path.isdir(output_path):
+        print("Output folder {} does not exist. Making it now".format(output_path))
+        Path(output_path).mkdir(parents=True)
+
+    ### naming of file (depending on magnet properties)
+
+    if flagIron == 1:
+        suffix_iron = "_WithIron"
+    else:
+        suffix_iron = "_NoIron"
+
+    if flagSelfField == 1:
+        suffix_self = "_WithSelfField"
+    else:
+        suffix_self = "_NoSelfField"
+
+    suffix_complete = suffix + suffix_iron + suffix_self
+    file_name_output = magnet_name + suffix_complete + ".map2d"
+
+    # Get data of (old) ROXIE file
+    content_ROXIE_file = open(map2d_file_name, "r").read().split("\n")
+
+    # Parse to ParserMap2d
+    NewMap2d = ParserMap2d.modify_map2d_ribbon_cable(map2d_file_name, geometry_ribbon_cable)
+    # Multiply 3rd,4th and 7th column to not have SI-Units
+    ParserMap2d.multiply_column_by_value(NewMap2d, 3, 1000)
+    ParserMap2d.multiply_column_by_value(NewMap2d, 4, 1000)
+    ParserMap2d.multiply_column_by_value(NewMap2d, 7, 1000000)
+    # Create .map2d file from ParserMap2d_
+    ParserMap2d.create_map2d_file(NewMap2d, header_line=content_ROXIE_file[0],
+                                  output_file_location=Path(output_path, file_name_output),
+                                  new_file_name=file_name_output,verbose=verbose)
+
+    return file_name_output
+#######################  Helper functions - END  #######################
\ No newline at end of file
