@@ -9,6 +9,9 @@ from typing import (Deque, Dict, FrozenSet, List, Optional, Sequence, Set, Tuple
 
 from steam_sdk.builders import geometricFunctions
 from steam_sdk.data import DataRoxieParser as pd
+# import matplotlib.pyplot as plt
+# import matplotlib.lines as lines
+# import matplotlib.patches as patches
 
 
 def arcCenter(C, iL, oL, iR, oR, diff_radius=None):
@@ -82,6 +85,7 @@ class ParserRoxie:
     dir_cadata: Path = None
 
     no: int = None
+    shift_flag: int = 0
     block: pd.Block = pd.Block()
     group: pd.Group = pd.Group()
     trans: pd.Trans = pd.Trans()
@@ -135,14 +139,14 @@ class ParserRoxie:
 
             for i in fileContent.split('\n'):  # content by row
                 fc = i.strip().strip(';')
-                if (fc.strip()[:2] != '--') and (fc != '') and (fc.strip() != 'HyperMesh') and (
-                        fc.strip()[:6] != 'Mirror') \
-                        and (fc.strip()[:5] != 'Lmesh') and (fc.strip()[:7] != 'include') and (fc.strip()[0] != '#'):
+                if (fc.strip()[:2] != '--' and fc != '' and fc.strip() != 'HyperMesh' and fc.strip()[:6] != 'Mirror'
+                        and fc.strip()[:7] != 'include' and fc.strip()[0] != '#'):
                     if '--' in fc:  # is there a comment?
                         fc = fc[:fc.find('--')].strip().strip(';')
 
                     for j in fc.strip().split(';'):  # row content
                         statement = j.strip().split('=')
+                        state0 = statement[0].strip()
 
                         if j[:3] == 'dv ':  # design variable
                             designVariables[statement[0][3:].strip()] = \
@@ -160,42 +164,38 @@ class ParserRoxie:
                                 arg = statement[1].strip(' []').split(',')
                                 A = [evalContent(arg[0].strip(), allGlobal), evalContent(arg[1].strip(), allGlobal)]
 
-                            data.key_points[statement[0].strip()] = pd.Coord(x=sigDig(A[0]), y=sigDig(A[1]))
+                            data.key_points[state0] = pd.Coord(x=sigDig(A[0]), y=sigDig(A[1]))
 
                         elif j[:2] == 'ln':  # hyper line
                             statement[1] = statement[1].strip()
                             if statement[1][:5] == 'Hyper':
                                 arguments = statement[1][10:-1].split(',')
+                                arg0 = arguments[0].strip()
+                                arg1 = arguments[1].strip()
                                 lineType = arguments[2].strip(' "')
 
                                 if lineType == 'Line':
-                                    data.hyper_lines[statement[0].strip()] = pd.HyperLine(
-                                        type='line', kp1=arguments[0].strip(), kp2=arguments[1].strip())
+                                    data.hyper_lines[state0] = pd.HyperLine(type='line', kp1=arg0, kp2=arg1)
 
                                 elif lineType[:6] == 'Corner':
                                     if lineType[6:] == 'Out':
-                                        arg = arguments[0].strip() + arguments[1].strip() + 'Out'  # intersection point
-                                        data.key_points[arg] = pd.Coord(
-                                            x=data.key_points[arguments[1].strip()].x,
-                                            y=data.key_points[arguments[0].strip()].y)
+                                        arg = arg0 + arg1 + 'Out'  # intersection point
+                                        data.key_points[arg] = pd.Coord(x=data.key_points[arg1].x,
+                                                                        y=data.key_points[arg0].y)
                                     else:
-                                        arg = arguments[0].strip() + arguments[1].strip() + 'In'
-                                        data.key_points[arg] = pd.Coord(x=data.key_points[arguments[0].strip()].x,
-                                                                        y=data.key_points[arguments[1].strip()].y)
+                                        arg = arg0 + arg1 + 'In'
+                                        data.key_points[arg] = pd.Coord(x=data.key_points[arg0].x,
+                                                                        y=data.key_points[arg1].y)
 
-                                    corners.append(statement[0].strip())
-                                    data.hyper_lines[statement[0].strip() + 'a'] = pd.HyperLine(
-                                        type='line', kp1=arguments[0].strip(), kp2=arg)
-                                    data.hyper_lines[statement[0].strip() + 'b'] = pd.HyperLine(
-                                        type='line', kp1=arg, kp2=arguments[1].strip())
+                                    corners.append(state0)
+                                    data.hyper_lines[state0 + 'b'] = pd.HyperLine(type='line', kp1=arg0, kp2=arg)
+                                    data.hyper_lines[state0 + 'a'] = pd.HyperLine(type='line', kp1=arg, kp2=arg1)
 
                                 elif lineType == 'Bar':
-                                    arg = [arguments[0].strip() + arguments[1].strip() + 'a',
-                                           arguments[0].strip() + arguments[1].strip() + 'b']  # rectangle corner points
-                                    A = [data.key_points[arguments[1].strip()].x,
-                                         data.key_points[arguments[1].strip()].y]
-                                    B = [data.key_points[arguments[0].strip()].x,
-                                         data.key_points[arguments[0].strip()].y]
+                                    arg = [arg0 + arg1 + 'a', arg0 + arg1 + 'b']  # rectangle corner points
+
+                                    A = [data.key_points[arg1].x, data.key_points[arg1].y]
+                                    B = [data.key_points[arg0].x, data.key_points[arg0].y]
                                     if A[0] - B[0] != 0.0:
                                         alpha = math.atan((A[1] - B[1]) / (B[0] - A[0]))
                                     else:  # is the bar horizontal?
@@ -207,7 +207,8 @@ class ParserRoxie:
                                         else:
                                             h = (B[0] - A[0]) / 2 / math.cos(alpha)
                                     else:
-                                        h = evalContent(arguments[3].strip(), {**designVariables, **scalarVariables,
+                                        h = (-1 * (B[0] < A[0]) + (B[0] > A[0])) * \
+                                            evalContent(arguments[3].strip(), {**designVariables, **scalarVariables,
                                                                                **data.key_points, **functions})
 
                                     data.key_points[arg[1]] = pd.Coord(x=sigDig(B[0] - h * math.sin(alpha)),
@@ -216,16 +217,15 @@ class ParserRoxie:
                                         x=sigDig(data.key_points[arg[1]].x + A[0] - B[0]),
                                         y=sigDig(data.key_points[arg[1]].y + A[1] - B[1]))
 
-                                    bars.append(statement[0].strip())
-                                    data.hyper_lines[statement[0].strip() + 'a'] = pd.HyperLine(
-                                        type='line', kp1=arguments[0].strip(), kp2=arg[1])
-                                    data.hyper_lines[statement[0].strip() + 'b'] = pd.HyperLine(
-                                        type='line', kp1=arg[1], kp2=arg[0])
-                                    data.hyper_lines[statement[0].strip() + 'c'] = pd.HyperLine(
-                                        type='line', kp1=arg[0], kp2=arguments[1].strip())
+                                    bars.append(state0)
+                                    data.hyper_lines[state0 + ('c' * (B[0] < A[0]) + 'a' * (B[0] > A[0]))] = \
+                                        pd.HyperLine(type='line', kp1=arg0, kp2=arg[1])
+                                    data.hyper_lines[state0 + 'b'] = pd.HyperLine(type='line', kp1=arg[1], kp2=arg[0])
+                                    data.hyper_lines[state0 + ('a' * (B[0] < A[0]) + 'c' * (B[0] > A[0]))] = \
+                                        pd.HyperLine(type='line', kp1=arg[0], kp2=arg1)
 
                                 elif lineType == 'Arc':
-                                    arg = [arguments[0].strip(), arguments[1].strip()]
+                                    arg = [arg0, arg1]
                                     if arguments[3].strip()[:2] == 'kp':
                                         arg.append(arguments[3].strip())
 
@@ -267,31 +267,26 @@ class ParserRoxie:
                                         data.key_points[arg[2]] = pd.Coord(x=sigDig(M[0] + sign * ss * math.cos(alpha)),
                                                                            y=sigDig(M[1] + sign * ss * math.sin(alpha)))
 
-                                    data.hyper_lines[statement[0].strip()] = pd.HyperLine(type='arc', kp1=arg[0],
-                                                                                          kp2=arg[1], kp3=arg[2])
+                                    data.hyper_lines[state0] = \
+                                        pd.HyperLine(type='arc', kp1=arg[0], kp2=arg[1], kp3=arg[2])
 
                                 elif lineType == 'Circle':
-                                    data.hyper_lines[statement[0].strip()] = pd.HyperLine(type='circle',
-                                                                                          kp1=arguments[0].strip(),
-                                                                                          kp2=arguments[1].strip())
+                                    data.hyper_lines[state0] = pd.HyperLine(type='circle', kp1=arg0, kp2=arg1)
 
                                 elif lineType == 'EllipticArc':
                                     allGlobal = {**designVariables, **scalarVariables, **data.key_points, **functions}
                                     arg = [evalContent(arguments[3].strip(), allGlobal),
                                            evalContent(arguments[4].strip(), allGlobal)]  # axes of the ellipse
-                                    data.hyper_lines[statement[0].strip()] = pd.HyperLine(type='ellipticArc',
-                                                                                          kp1=arguments[0].strip(),
-                                                                                          kp2=arguments[1].strip(),
-                                                                                          arg1=arg[0], arg2=arg[1])
+                                    data.hyper_lines[state0] = \
+                                        pd.HyperLine(type='ellipticArc', kp1=arg0, kp2=arg1, arg1=arg[0], arg2=arg[1])
 
                                 else:
                                     print(arguments[2].strip() + ' needs to be implemented')
 
                             elif statement[1][:4] == 'Line':
                                 arguments = statement[1][5:-1].split(',')
-                                data.hyper_lines[statement[0].strip()] = pd.HyperLine(type='line',
-                                                                                      kp1=arguments[0].strip(),
-                                                                                      kp2=arguments[1].strip())
+                                data.hyper_lines[state0] = pd.HyperLine(type='line', kp1=arguments[0].strip(),
+                                                                        kp2=arguments[1].strip())
 
                             else:
                                 print(statement[1][:statement[1].find('(')] + ' needs to be implemented')
@@ -300,28 +295,46 @@ class ParserRoxie:
                             statement[1] = statement[1].strip()
                             arguments = statement[1][10:-1].split(',')
                             arg = []
+                            pre_line = []
                             for k in range(len(arguments) - 1):
                                 name = arguments[k].strip()
                                 if name in corners:  # 2 lines are introduced for corners
                                     arg.extend([name + 'a', name + 'b'])
                                 elif name in bars:  # 3 lines are introduced for bars
-                                    arg.extend([name + 'a', name + 'b', name + 'c'])
+                                    if (data.hyper_lines[name + 'a'].kp1 in pre_line or
+                                            data.hyper_lines[name + 'a'].kp2 in pre_line):
+                                        arg.extend([name + 'a', name + 'b', name + 'c'])
+                                    else:
+                                        arg.extend([name + 'c', name + 'b', name + 'a'])
                                 else:
                                     arg.append(name)
+                                pre_line = [data.hyper_lines[arg[-1]].kp1, data.hyper_lines[arg[-1]].kp2]
 
-                            data.hyper_areas[statement[0].strip()] = pd.HyperArea(material=arguments[-1].strip(),
-                                                                                  lines=arg)
+                            data.hyper_areas[state0] = pd.HyperArea(material=arguments[-1].strip(), lines=arg)
 
                         elif j[:2] == 'BH':
                             print('BH')
 
                         elif j[:11] == 'HyperHoleOf':
-                            arguments = statement[0].strip()[12:-1].split(',')
-                            data.hyper_holes[len(data.hyper_holes) + 1] = pd.HyperHole(areas=[arguments[0].strip(),
-                                                                                              arguments[1].strip()])
+                            arguments = state0[12:-1].split(',')
+                            data.hyper_holes[len(data.hyper_holes) + 1] = \
+                                pd.HyperHole(areas=[arguments[0].strip(), arguments[1].strip()])
+
+                        elif j[:5] == 'Lmesh':
+                            arguments = state0[6:-1].split(',')
+                            name = arguments[0].strip()
+                            if name in corners:
+                                data.hyper_lines[name + 'a'].elements = round(int(arguments[1].strip()) / 2)
+                                data.hyper_lines[name + 'b'].elements = round(int(arguments[1].strip()) / 2)
+                            elif name in bars:
+                                data.hyper_lines[name + 'a'].elements = round(int(arguments[1].strip()) / 3)
+                                data.hyper_lines[name + 'b'].elements = round(int(arguments[1].strip()) / 3)
+                                data.hyper_lines[name + 'c'].elements = round(int(arguments[1].strip()) / 3)
+                            else:
+                                data.hyper_lines[name].elements = int(arguments[1].strip())
 
                         else:  # scalar variables
-                            scalarVariables[statement[0].strip()] = \
+                            scalarVariables[state0] = \
                                 evalContent(statement[1].strip(),
                                             {**designVariables, **scalarVariables, **data.key_points, **functions})
             if isinstance(f, Path):
@@ -497,15 +510,19 @@ class ParserRoxie:
                                                            alph=float(rowSplitStr[3]), bet=float(rowSplitStr[4]),
                                                            string=str(rowSplitStr[5]), act=int(rowSplitStr[6]),
                                                            bcs=list(map(int, rowSplitStr[7:-1])))
+                    if data.transs[rowSplitStr[0]].string == 'SHIFT2':
+                        self.shift_flag += 1
                 else:  # block
                     block_nr = int(rowSplitStr[0])
                     blocks_list = []
+
                     for group in data.groups:
                         blocks_list += data.groups[group].blocks
                         if block_nr in data.groups[group].blocks:
                             group_nr = int(group)
                             symm = data.groups[group].symm
-                            if not data.transs:
+
+                            if not data.transs or self.shift_flag <= 1:
                                 coil = 1
                             else:
                                 groups_list = []
@@ -680,7 +697,7 @@ class ParserRoxie:
                 print('typexy = {}: One coil.'.format(self.group.typexy))
 
             for additionalBlock in self.group.blocks:
-                nOriginalBlocks = nb
+                # nOriginalBlocks = nb
                 if verbose:
                     print('additionalBlock = {}'.format(additionalBlock))
 
@@ -804,7 +821,7 @@ class ParserRoxie:
 
         # Apply symmetry conditions to blocks
         for g, no in enumerate(data.groups):
-            if not data.transs:
+            if not data.transs or self.shift_flag <= 1:
                 self.no = 1
             else:
                 for trans in data.transs:
@@ -1230,7 +1247,18 @@ class ParserRoxie:
             #     block.block_corners.oR = pd.Coord(x=sigDig(xCable[2]), y=sigDig(yCable[2]))  # 3
             # else:
             #     pass
-            if c == self.block.nco - 1:
+
+            # ax.add_patch(patches.Circle((coil.bore_center.x, coil.bore_center.y), radius=radius, color='b'))
+            # self.ax.add_line(lines.Line2D([xCable[0], xCable[3]], [yCable[0], yCable[3]], color='red'))
+            # self.ax.add_line(lines.Line2D([xCable[3], xCable[2]], [yCable[3], yCable[2]], color='red'))
+            # self.ax.add_line(lines.Line2D([xCable[2], xCable[1]], [yCable[2], yCable[1]], color='red'))
+            # self.ax.add_line(lines.Line2D([xCable[1], xCable[0]], [yCable[1], yCable[0]], color='red'))
+            if c == self.block.nco - 1 and c == 0:
+                corner0 = [xCable[0], yCable[0]]  # 1
+                corner3 = [xCable[3], yCable[3]]  # 4
+                corner1 = [xCable[1], yCable[1]]  # 2
+                corner2 = [xCable[2], yCable[2]]  # 3
+            elif c == self.block.nco - 1:
                 corner0 = [xCable[0], yCable[0]]  # 1
                 corner3 = [xCable[3], yCable[3]]  # 4
             elif c == 0:
@@ -1277,6 +1305,9 @@ class ParserRoxie:
                     # yS.append(yStrand)
                     # # iS.append(currentStrand)
 
+        # Compute two new corners to make concentric blocks by intersecting a circle (having a radius equal to the
+        # distance between the bore center and the starting inner/outer block corner) with the straight line
+        # passing through the ending block corners that need to be replaced
         new_corners_inner = geometricFunctions.intersectLineCircle(
             geometricFunctions.findLineThroughTwoPoints(corner0, corner3),
             [radius, coil.bore_center.x, coil.bore_center.y])
@@ -1331,6 +1362,10 @@ class ParserRoxie:
         if blocks:
             self.roxieData.coil.blocks = blocks
 
+        # plt.figure(figsize=(10, 10))
+        # self.ax = plt.axes()
+        # self.ax.set_xlim(-0.2, 0.2)
+        # self.ax.set_ylim(-0.2, 0.2)
         for no in self.roxieData.coil.blocks:
             self.no = int(no)
             self.block = self.roxieData.coil.blocks[no]
@@ -1346,7 +1381,7 @@ class ParserRoxie:
 
         # if verbose:
         #     print('Total number of conductors (half-turns): {}'.format(len(self.xPos)))
-
+        # plt.show()
         return self.data.coil
 
     def getWedgePositions(self, coil: pd.CoilData = None, verbose: bool = False):
